/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/

#include "StreamView/stream_editor.h"
#include "StreamModel/service_xspf.h"

#include "MediaItem/mediaitem.h"
#include "Dialog/addstreamdialog.h"
#include "debug.h"

#include <QtGui>

/*
********************************************************************************
*                                                                              *
*    Class StreamEditor                                                        *
*                                                                              *
********************************************************************************
*/
StreamEditor::StreamEditor(QWidget *parent) : QDialog(parent)
{
    // init tree widget
    m_treeWidget = new QTreeWidget(this);
    m_treeWidget->setFrameShape(QFrame::NoFrame);
    m_treeWidget->setSortingEnabled(false);
    m_treeWidget->setDragDropMode(QAbstractItemView::InternalMove);
    m_treeWidget->setDragEnabled(true);
    m_treeWidget->setAcceptDrops(true);
    m_treeWidget->setDropIndicatorShown(true);

    QTreeWidgetItem *headerItem = m_treeWidget->headerItem();
    headerItem->setText(0, tr("Category/Title"));
    headerItem->setText(1, tr("Url"));

    connect(m_treeWidget->model(), SIGNAL(dataChanged(const QModelIndex&,const QModelIndex&) ), this, SLOT(slot_on_data_changed()));

    // GUI
    create_ui();

    is_data_loading = false;
    ui_save_action->setEnabled(false);
}

StreamEditor::~StreamEditor()
{
    Debug::debug() << "StreamEditor -> closing ...";
}


/*******************************************************************************
 ui part
*******************************************************************************/
void StreamEditor::create_ui()
{
    //! ui part
    this->setModal(true);
    this->setWindowTitle(tr("Stream Editor"));
    this->resize(500,620);

    //In Linux with KDE this code make a window without an close and minimize
    // and maximize buttons in title bar.
    this->setWindowFlags( Qt::Dialog | Qt::WindowTitleHint );

    // save action
    ui_save_action = new QAction(QIcon(":/images/save-as-32x32.png"), tr("&Save stream list"), this);
    connect(ui_save_action, SIGNAL(triggered(bool)), this, SLOT(slot_save_stream_list()));

    // set toolbar & action
    QToolBar* toolbar = new QToolBar(this);
    //toolbar->addAction( QIcon(":/images/save-as-32x32.png"), tr("&Save stream list"), this, SLOT(slot_save_stream_list()));
    toolbar->addAction( ui_save_action );
    toolbar->addAction( QIcon(":/images/document-new-48x48.png"), tr("&Add new entry"), this, SLOT(slot_add_entry()));
    toolbar->addAction( QIcon::fromTheme("edit-delete"), tr("&Remove entry stream list"), this, SLOT(slot_delete_entry()));

    // name bar
    ui_name_bar = new QWidget(this);

    QGridLayout* gridLayout = new QGridLayout(ui_name_bar);
    gridLayout->setContentsMargins(4, 0, 0, 0);
    gridLayout->setSpacing(4);

    QLabel* label = new QLabel(tr("name"));

    ui_line_edit = new ExLineEdit(this);
    ui_line_edit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
    ui_line_edit->clearFocus();
    ui_line_edit->setInactiveText(tr("enter a name for list"));

    gridLayout->addWidget(label, 0, 0, 1, 1);
    gridLayout->addWidget(ui_line_edit, 0, 1, 1, 1);


    //main frame
    QFrame* frame = new QFrame();
    frame->setFrameShape(QFrame::StyledPanel);
    frame->setFrameShadow(QFrame::Sunken);

    QVBoxLayout *vl0 = new QVBoxLayout(frame);
    vl0->setSpacing(0);
    vl0->setContentsMargins(0, 0, 0, 0);
    vl0->addWidget(m_treeWidget);

    QVBoxLayout *vl1 = new QVBoxLayout(this);
    vl1->setSpacing(0);
    vl1->setContentsMargins(0, 0, 0, 0);
    vl1->addWidget(toolbar);
    vl1->addWidget(ui_name_bar);
    vl1->addWidget(frame);


    ui_status_bar  = new QStatusBar(this);
    vl1->addWidget(ui_status_bar);

    //! QDialogButtonBox
    QDialogButtonBox* ui_buttonBox = new QDialogButtonBox();
    ui_buttonBox->setOrientation(Qt::Horizontal);
    ui_buttonBox->setStandardButtons(QDialogButtonBox::Close);
    vl1->addWidget(ui_buttonBox);
    QObject::connect(ui_buttonBox, SIGNAL(rejected()), this, SLOT(slot_on_buttonBox_rejected()));

    // avoid focus to be on line edit
    this->setFocus(Qt::OtherFocusReason);

}

void StreamEditor::slot_on_buttonBox_rejected()
{
    this->setResult(QDialog::Rejected);
    QDialog::reject();
    this->close();
}


/*******************************************************************************
 PUBLIC load_stream_file
*******************************************************************************/
void StreamEditor::load_stream_file(const QString& path, bool isFavorite)
{
    is_data_loading = true;
    m_filepath      = path;

    XspfStreams xspf(path);
    xspf.load();

    ui_line_edit->setText( xspf.playlist_name() );
    if(isFavorite)
      ui_name_bar->hide();

    QString category = QString();
    QTreeWidgetItem* cat_item;

    foreach(MEDIA::TrackPtr mi, xspf.streams() )
    {
      if(mi->categorie != category )
      {
        category = mi->categorie;

        cat_item = new QTreeWidgetItem(0);
        cat_item->setFlags(cat_item->flags() | Qt::ItemIsEditable | Qt::ItemIsDropEnabled);
        cat_item->setFlags(cat_item->flags() & ~Qt::ItemIsDragEnabled);

        QFont font = cat_item->font(0);
        font.setBold(true);
        cat_item->setFont(0 , font);

        cat_item->setData(0, Qt::EditRole,category);

        m_treeWidget->addTopLevelItem(cat_item);
        //Debug::debug() << "category found : " << category;
      }

      QTreeWidgetItem* stream_item = new QTreeWidgetItem( cat_item );
      stream_item->setFlags(stream_item->flags() | Qt::ItemIsEditable | Qt::ItemIsDragEnabled);
      stream_item->setFlags(stream_item->flags() & ~Qt::ItemIsDropEnabled);
      stream_item->setData(0, Qt::EditRole, mi->name );
      stream_item->setData(1, Qt::EditRole, mi->url );
    }

    ui_status_bar->showMessage(tr("stream file loaded OK !"), 2500);
    is_data_loading = false;
}

/*******************************************************************************
 slot_save_stream_list
*******************************************************************************/
void StreamEditor::slot_save_stream_list()
{
    // get media item
    QList<MEDIA::TrackPtr> media_list;

    for (int i = 0; i < m_treeWidget->topLevelItemCount(); i++)
    {
      QTreeWidgetItem *category_item =  m_treeWidget->topLevelItem ( i );

      QString category = category_item->data(0, Qt::EditRole).toString();
      //Debug::debug() << "StreamEditor::slot_save_stream_list # category : " <<  category;

      for (int j = 0; j < category_item->childCount(); j++)
      {
          MEDIA::TrackPtr media = MEDIA::TrackPtr(new MEDIA::Track());

          QTreeWidgetItem* stream_item = category_item->child ( j ) ;

          media->setType(TYPE_STREAM);
          media->categorie = category;
          media->name      = stream_item->data(0, Qt::EditRole).toString();
          media->url       = stream_item->data(1, Qt::EditRole).toString();

          //Debug::debug() << "StreamEditor::slot_save_stream_list # name : " <<  media->data.name;
          //Debug::debug() << "StreamEditor::slot_save_stream_list # url : " <<  media->data.url;

          media_list << media;
      }
    }

    // save to file
    qSort(media_list.begin(), media_list.end(), MEDIA::compareStreamCategorie);

    XspfStreams xspf(m_filepath);
    xspf.setStreams(media_list);
    xspf.set_playlist_name( ui_line_edit->text() );
    xspf.save();

    ui_status_bar->showMessage(tr("list is saved"), 2000);
    emit stream_list_saved();
}


/*******************************************************************************
 slot_delete_entry
*******************************************************************************/
void StreamEditor::slot_delete_entry()
{
    QList<QTreeWidgetItem *> selections = m_treeWidget->selectedItems();
    if(selections.isEmpty())
      return;

    QTreeWidgetItem* item = selections.first();
    if(item)
      delete item;
}

/*******************************************************************************
 slot_add_entry
*******************************************************************************/
void StreamEditor::slot_add_entry()
{
    AddStreamDialog dialog(this,true);

    if(dialog.exec() == QDialog::Accepted)
    {
      const QString category = dialog.category();
      const QString name     = dialog.name();
      const QString url      = dialog.url();

      if( !category.isEmpty() && !name.isEmpty() && ! url.isEmpty() )
      {
        // find if same category already exist
        QList<QTreeWidgetItem *> list = m_treeWidget->findItems( category, Qt::MatchContains );

        QTreeWidgetItem* cat_item = 0;
        if(!list.isEmpty())
          cat_item = list.first();

        if (!cat_item)
        {
            Debug::debug() << "StreamEditor::slot_add_entry no category find";

            // insert category
            cat_item = new QTreeWidgetItem(0);
            cat_item->setFlags(cat_item->flags() | Qt::ItemIsEditable | Qt::ItemIsDropEnabled);
            cat_item->setFlags(cat_item->flags() & ~Qt::ItemIsDragEnabled);

            QFont font = cat_item->font(0);
            font.setBold(true);
            cat_item->setFont(0 , font);

            cat_item->setData(0, Qt::EditRole,category);

            m_treeWidget->addTopLevelItem(cat_item);
        }

        //Debug::debug() << "StreamEditor::slot_add_entry add stream item  cat_item =" << cat_item;

        // insert stream item
        QTreeWidgetItem* stream_item = new QTreeWidgetItem( cat_item );
        stream_item->setFlags(stream_item->flags() | Qt::ItemIsEditable | Qt::ItemIsDragEnabled);
        stream_item->setFlags(stream_item->flags() & ~Qt::ItemIsDropEnabled);
        stream_item->setData(0, Qt::EditRole, name );
        stream_item->setData(1, Qt::EditRole, url );
      }
      else
      {
        QMessageBox message;
        message.setText(QString(tr("Please fill all requested fields")).toAscii());
        message.setStandardButtons(QMessageBox::Ok);
        message.resize(445, 120);
        message.exec();
      }
    }
}

/*******************************************************************************
 slot_on_data_changed
*******************************************************************************/
void StreamEditor::slot_on_data_changed()
{
    if(is_data_loading)
      return;

    //Debug::debug() << "StreamEditor::slot_on_data_changed";
    ui_status_bar->showMessage(tr("list has been updated,...save it or close to cancel"));
    ui_save_action->setEnabled(true);
}

